/*

	Predator.h
	
	adopted for stampTV from EffectTV source by Stephan Aßmus
	
	original copyright notice follows:

 * EffecTV - Realtime Digital Video Effector
 * Copyright (C) 2001 FUKUCHI Kentarou


*/

#ifndef _PREDATOR_H_
#define _PREDATOR_H_

#include "VideoPlugin.h"

#include <Rect.h>

#define MAGIC_THRESHOLD 40

class BBitmap;

class PredatorPluginType : public VideoPluginType
{
	public:
							PredatorPluginType(VideoPluginsHost * host, image_id id);
		virtual				~PredatorPluginType();

		virtual const char*	ID();
		virtual uint32		Flags();
		virtual uint32		Version();
		virtual const char*	Name();
		virtual const char*	Author();
		virtual const char*	AboutString();
		virtual status_t	About();
		virtual int32		GetVerticesProperties(const vertice_properties ** ap);
		virtual VideoPluginEngine * InstanciateVideoPluginEngine();

	private:
};

class PredatorEngine : public VideoPluginEngine
{
	public:
							PredatorEngine(VideoPluginType * plugin);
		virtual				~PredatorEngine();
		friend class PredatorPluginType;

		virtual status_t	SetConfig(BMessage & config);
		virtual status_t	GetConfig(BMessage & config);
		virtual BView*		ConfigureView();

		// UseFormat will be called before ApplyEffect is first called and when the image format changes.
		// No need to monitor the BBitmap format.
		virtual status_t	UseFormat(media_raw_video_format & format, int which_work_mode);
		virtual status_t	ApplyEffect(BBitmap * frame_in, BBitmap * frame_out, int64 frame_count, bool skipped_frames);

	private:
		void TakeBackground(BBitmap *src);

		int					format;
		unsigned	int		*bgimage;
		unsigned	char	*bgvalue;
		unsigned	char	*buffer;
		int32				width;
		int32				height;

		BRect				bounds;
		color_space			space;
		volatile	bool	initialized;
		volatile	bool	bg_taken;
		volatile	int32	pics_taken;
};

#endif // _PREDATOR_H_
