/*
 * EffecTV - Realtime Digital Video Effector
 * Copyright (C) 2001 FUKUCHI Kentarou
 *
 * yuv.cpp: YUV(YCbCr) color system utilities
 *
 */

#include <math.h>
#include "utils.h"

/*
 * conversion from YUV to RGB
 *   r = 1.164*(y-16) + 1.596*(v-128);
 *   g = 1.164*(y-16) - 0.813*(v-128) - 0.391*(u-128);
 *   b = 1.164*(y-16)                 + 2.018*(u-128);
 * conversion from RGB to YUV
 *   y =  0.257*r + 0.504*g + 0.098*b + 16
 *   u = -0.148*r - 0.291*g + 0.439*b + 128
 *   v =  0.439*r - 0.368*g - 0.071*b + 128
 */

int YtoRGB[256];
int VtoR[256], VtoG[256];
int UtoG[256], UtoB[256];
int RtoY[256], RtoU[256], RtoV[256];
int GtoY[256], GtoU[256], GtoV[256];
int BtoY[256],            BtoV[256];

int yuvTableInit()
{
	static int initialized = 0;

	if(!initialized) {
		for(int i = 0; i < 256; i++)
		{
			YtoRGB[i] =  (int)(1.164 * (i - 16));
			VtoR[i] =  (int)(1.596 * (i - 128));
			VtoG[i] = (int)(-0.813 * (i - 128));
			UtoG[i] = (int)(-0.391 * (i - 128));
			UtoB[i] =  (int)(2.018 * (i - 128));
			RtoY[i] =  (int)(0.257 * i);
			RtoU[i] = (int)(-0.148 * i);
			RtoV[i] =  (int)(0.439 * i);
			GtoY[i] =  (int)(0.504 * i);
			GtoU[i] = (int)(-0.291 * i);
			GtoV[i] = (int)(-0.368 * i);
			BtoY[i] =  (int)(0.098 * i);
			BtoV[i] = (int)(-0.071 * i);
		}
		initialized = 1;
	}
	return 0;
}

unsigned char RGBtoY(int rgb)
{
	int i;

	i = RtoY[(rgb >> 16) & 0xff];
	i += GtoY[(rgb >> 8) & 0xff];
	i += BtoY[rgb & 0xff];
	i += 16;
	if (i > 255)
		return 255;
	return i;
}
