/*

	QuarksFlip.cpp
	
	© 2001, Stephan Aßmus, All Rights Reserved.
	
	idea stolen from EffecTV by FUKUCHI Kentarou
	
	based on HorizontalFlip.cpp sample code provided by Georges-Edouard Berenger

*/

#include "Quarks.h"

#include <Bitmap.h>
#include <SupportDefs.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

status_t GetVideoPlugins(TList<VideoPluginType> & plugin_list, VideoPluginsHost * host, image_id id)
{
	if (host->APIVersion() < VideoPluginsHost::VAPI_MINIMAL_API_VERSION)
		return VAPI_HOST_TOO_OLD;

	plugin_list.AddItem(new QuarksPluginType(host, id));
	return B_OK;
}

QuarksPluginType::QuarksPluginType(VideoPluginsHost * host, image_id id)
				: VideoPluginType(host, id)
{
}

QuarksPluginType::~QuarksPluginType()
{
}

const char* QuarksPluginType::ID()
{
	return "Quarks";
}

uint32 QuarksPluginType::Flags()
{
	return 0;
}

uint32 QuarksPluginType::Version()
{
	return 001;
}

const char* QuarksPluginType::Name()
{
	return "Quarks";
}

const char* QuarksPluginType::Author()
{
	return "stippi";
}

const char* QuarksPluginType::AboutString()
{
	return "Quarks by Stephan Aßmus, original idea by FUKUCHI Kentarou";
}

status_t QuarksPluginType::About()
{
	return B_OK;
}

static const vertice_properties support[] =
{
	{	ANY_COLOR_SPACE,
		ANY_COLOR_SPACE,
		1.0,
		VAPI_PROCESS_TO_DISTINCT, //| VAPI_PROCESS_IN_PLACE | VAPI_PROCESS_TO_OVERLAY,
	},
};

int32 QuarksPluginType::GetVerticesProperties(const vertice_properties ** ap)
{
	*ap = support;
	return sizeof(support) / sizeof(vertice_properties);
}

VideoPluginEngine * QuarksPluginType::InstanciateVideoPluginEngine()
{
	return new QuarksEngine(this);
}

//-------------------------------------------------------------------------

QuarksEngine::QuarksEngine(VideoPluginType * plugin)
			: VideoPluginEngine(plugin),
			planes(NULL),
			initialized(false)
{
}

QuarksEngine::~QuarksEngine()
{
	if (planes)
	{
		for (int32 i = 0; i < PLANES; i++)
			if (planes[i])
				delete planes[i];
		delete[] planes;
	}
}

status_t QuarksEngine::SetConfig(BMessage & config)
{
	return B_OK;
}

status_t QuarksEngine::GetConfig(BMessage & config)
{
	return B_OK;
}

BView* QuarksEngine::ConfigureView()
{
	return NULL;
}

status_t QuarksEngine::UseFormat(media_raw_video_format & format, int which_work_mode)
{
	initialized = false;
	if (planes)
	{
		for (int32 i = 0; i < PLANES; i++)
			if (planes[i])
				delete planes[i];
		delete[] planes;
	}
	planes = new BBitmap*[PLANES];
	bounds.Set(0.0, 0.0, (float)format.display.line_width - 1, (float)format.display.line_count - 1);
	space = format.display.format;
	for (int32 i = 0; i < PLANES; i++)
	{
		planes[i] = new BBitmap(bounds,	space);
		if (!planes[i] || !planes[i]->IsValid())
			return B_NO_MEMORY;
	}
	active_plane = 0;
	initialized = true;
	return B_OK;
}

status_t QuarksEngine::ApplyEffect(BBitmap * frame_in,
								   BBitmap * frame_out,
								   int64 frame_count,
								   bool skipped_frames)
{
//bool profile = frame_count % 50 == 0;

	if (!initialized)
		return B_NO_INIT;
		
	if (!frame_in || !frame_out)
		return B_ERROR;

	if (frame_in->Bounds() != bounds)
		return B_ERROR;
	
	if (frame_in->BytesPerRow() != planes[0]->BytesPerRow())
		return B_ERROR;

	if (space != frame_out->ColorSpace())
		return B_ERROR;

	if (skipped_frames)
		return B_OK;

//bigtime_t now = system_time();
	// store incoming bitmap in one of our buffers
	if (active_plane == PLANES)
		active_plane = 0;
	// DO NOT USE memcpy!!! IT  *H*O*R*R*R*R*I*B*L*Y*  SLOW WITH OVERLAYS!!!
//	memcpy(planes[active_plane]->Bits(), frame_in->Bits(), frame_in->BitsLength());
	long * S = (long*) frame_in->Bits();
	long * D = (long*) planes[active_plane]->Bits();
	int l = frame_in->BitsLength() / 4;
	while (l >= 16) {
		*D = *S; D[1] = S[1]; D[2] = S[2]; D[3] = S[3]; D[4] = S[4]; D[5] = S[5]; D[6] = S[6]; D[7] = S[7];
		D[8] = S[8]; D[9] = S[9]; D[10] = S[10]; D[11] = S[11]; D[12] = S[12]; D[13] = S[13]; D[14] = S[14]; D[15] = S[15];
		S += 16; D += 16; l -= 16;
	}
	while (l >= 4) {
		*D = *S; D[1] = S[1]; D[2] = S[2]; D[3] = S[3];
		S += 4; D += 4; l -= 4;
	}
	while (l >= 1) {
		*D = *S;
		S += 1; D += 1; l -= 1;
	}
	active_plane++;

//if (profile)
//{
//printf("memcpy: %Ld\n", system_time() - now);
//now = system_time();
//}
	// randomly put pixels into frame_out - from all stored bitmaps
	switch (space)
	{
		case B_RGB32:
		case B_RGBA32:
		{
			// generate random number table
			int32 RANDOM_NUMBERS = rand() % (MAX_RANDOM_NUMBERS - MIN_RANDOM_NUMBERS) + MIN_RANDOM_NUMBERS;
			int32 *random_table[RANDOM_NUMBERS];
			for (int32 i = 0; i < RANDOM_NUMBERS; i++)
				random_table[i] = (int32 *)planes[rand() % PLANES]->Bits();
			// apply effect
			int32 *out_bits = (int32 *)frame_out->Bits();
			int32 pixels = frame_out->BitsLength() / sizeof(int32);
			for (int32 i = 0; i < pixels; )
			{
				int32** rtable = random_table + min_c(RANDOM_NUMBERS, pixels - i);
				while (rtable > random_table)
				{
					rtable--;
					*out_bits = (*rtable)[i];
					out_bits++;
					i++;
				}
			}
			break;
		}
		case B_RGB16:
		case B_RGB15:
		{
			// generate random number table
			int32 RANDOM_NUMBERS = rand() % (MAX_RANDOM_NUMBERS - MIN_RANDOM_NUMBERS) + MIN_RANDOM_NUMBERS;
			int16 *random_table[RANDOM_NUMBERS];
			for (int32 i = 0; i < RANDOM_NUMBERS; i++)
				random_table[i] = (int16 *)planes[rand() % PLANES]->Bits();
			// apply effect
			int16 *out_bits = (int16 *)frame_out->Bits();
			int32 pixels = frame_out->BitsLength() / sizeof(int16);
			for (int32 i = 0; i < pixels; )
			{
				int16** rtable = random_table + min_c(RANDOM_NUMBERS, pixels - i);
				while (rtable > random_table)
				{
					rtable--;
					*out_bits = (*rtable)[i];
					out_bits++;
					i++;
				}
			}			break;
		}
		case B_CMAP8:
		case B_GRAY8:
		{
			// generate random number table
			int32 RANDOM_NUMBERS = rand() % (MAX_RANDOM_NUMBERS - MIN_RANDOM_NUMBERS) + MIN_RANDOM_NUMBERS;
			int8 *random_table[RANDOM_NUMBERS];
			for (int32 i = 0; i < RANDOM_NUMBERS; i++)
				random_table[i] = (int8 *)planes[rand() % PLANES]->Bits();
			// apply effect
			int8 *out_bits = (int8 *)frame_out->Bits();
			int32 pixels = frame_out->BitsLength() / sizeof(int8);
			for (int32 i = 0; i < pixels; )
			{
				int8** rtable = random_table + min_c(RANDOM_NUMBERS, pixels - i);
				while (rtable > random_table)
				{
					rtable--;
					*out_bits = (*rtable)[i];
					out_bits++;
					i++;
				}
			}
			break;
		}
		default:
		{
			// leave bitmap unchanged
			long * S = (long*) frame_in->Bits();
			long * D = (long*) frame_out->Bits();
			int l = frame_in->BitsLength() / 4;
			while (l >= 16) {
				*D = *S; D[1] = S[1]; D[2] = S[2]; D[3] = S[3]; D[4] = S[4]; D[5] = S[5]; D[6] = S[6]; D[7] = S[7];
				D[8] = S[8]; D[9] = S[9]; D[10] = S[10]; D[11] = S[11]; D[12] = S[12]; D[13] = S[13]; D[14] = S[14]; D[15] = S[15];
				S += 16; D += 16; l -= 16;
			}
			while (l >= 4) {
				*D = *S; D[1] = S[1]; D[2] = S[2]; D[3] = S[3];
				S += 4; D += 4; l -= 4;
			}
			while (l >= 1) {
				*D = *S;
				S += 1; D += 1; l -= 1;
			}
			break; // should we return an error?
		}
	}
//	
//if (profile)
//printf("effect: %Ld\n", system_time() - now);
	return B_OK;
}

