/*

	QuarksFlip.h
	
	adopted for stampTV from EffectTV source by Stephan Aßmus
	
	original copyright notice follows:

 * EffecTV - Realtime Digital Video Effector
 * Copyright (C) 2001 FUKUCHI Kentarou
 *
 * quark.c: particle effect


*/

#ifndef _QUARKS_H_
#define _QUARKS_H_

#include "VideoPlugin.h"

#include <Rect.h>

#define PLANES 4
#define MAX_RANDOM_NUMBERS 50
#define MIN_RANDOM_NUMBERS 30

class BBitmap;

class QuarksPluginType : public VideoPluginType
{
	public:
							QuarksPluginType(VideoPluginsHost * host, image_id id);
		virtual				~QuarksPluginType();

		virtual const char*	ID();
		virtual uint32		Flags();
		virtual uint32		Version();
		virtual const char*	Name();
		virtual const char*	Author();
		virtual const char*	AboutString();
		virtual status_t	About();
		virtual int32		GetVerticesProperties(const vertice_properties ** ap);
		virtual VideoPluginEngine * InstanciateVideoPluginEngine();

	private:
};

class QuarksEngine : public VideoPluginEngine
{
	public:
							QuarksEngine(VideoPluginType * plugin);
		virtual				~QuarksEngine();
		friend class QuarksPluginType;

		virtual status_t	SetConfig(BMessage & config);
		virtual status_t	GetConfig(BMessage & config);
		virtual BView*		ConfigureView();

		// UseFormat will be called before ApplyEffect is first called and when the image format changes.
		// No need to monitor the BBitmap format.
		virtual status_t	UseFormat(media_raw_video_format & format, int which_work_mode);
		virtual status_t	ApplyEffect(BBitmap * frame_in, BBitmap * frame_out, int64 frame_count, bool skipped_frames);

	private:
		BBitmap				**planes;
		BRect				bounds;
		color_space			space;
		int32				active_plane;
		volatile	bool	initialized;
};

#endif // _QUARKS_H_
